@@ -12,7 +12,6 @@ module Assistant.WebApp.Configurators.Pairing where
 
 import Assistant.Pairing
 import Assistant.WebApp.Common
-import Assistant.WebApp.Configurators
 import Assistant.Types.Buddies
 import Annex.UUID
 #ifdef WITH_PAIRING
@@ -22,7 +21,6 @@ import Assistant.Ssh
 import Assistant.Alert
 import Assistant.DaemonStatus
 import Utility.Verifiable
-import Utility.Network
 #endif
 #ifdef WITH_XMPP
 import Assistant.XMPP.Client
@@ -32,6 +30,7 @@ import Network.Protocol.XMPP
 import Assistant.Types.NetMessager
 import Assistant.NetMessager
 import Assistant.WebApp.RepoList
+import Assistant.WebApp.Configurators
 import Assistant.WebApp.Configurators.XMPP
 #endif
 import Utility.UserInfo
@@ -266,8 +265,8 @@ data InputSecret = InputSecret { secretText :: Maybe Text }
 promptSecret :: Maybe PairMsg -> (Text -> Secret -> Widget) -> Handler Html
 promptSecret msg cont = pairPage $ do
 	((result, form), enctype) <- liftH $
-		runFormPostNoToken $ renderBootstrap $
-			InputSecret <$> aopt textField "Secret phrase" Nothing
+		runFormPostNoToken $ renderBootstrap3 bootstrapFormLayout $
+			InputSecret <$> aopt textField (bfs "Secret phrase") Nothing
 	case result of
 		FormSuccess v -> do
 			let rawsecret = fromMaybe "" $ secretText v
