@@ -118,20 +118,37 @@ openFileBrowser = do
 	path <- liftAnnex $ fromRepo Git.repoPath
 #ifdef darwin_HOST_OS
 	let cmd = "open"
-	let params = [Param path]
+	let p = proc cmd [path]
 #else
 #ifdef mingw32_HOST_OS
+	{- Changing to the directory and then opening . works around
+	 - spaces in directory name, etc. -}
 	let cmd = "cmd"
-	let params = [Param $ "/c start " ++ path]
+	let p = (proc cmd ["/c start ."]) { cwd = Just path }
 #else
 	let cmd = "xdg-open"
-	let params = [Param path]
+	let p = proc cmd [path]
 #endif
 #endif
 	ifM (liftIO $ inPath cmd)
 		( do
-			void $ liftIO $ forkIO $ void $
-				boolSystem cmd params
+			let run = void $ liftIO $ forkIO $ void $
+				createProcess p
+			run
+#ifdef mingw32_HOST_OS
+			{- On windows, if the file browser is not
+			 - already open, it comes up below the
+			 - web browser when started. 
+			 -
+			 - Running it a second time brings it
+			 - to the foreground.
+			 -
+			 - Seems to need a delay long enough for the file
+			 - browser to be open in order to work. Here 1
+			 - second. -}
+			liftIO $ threadDelay 1000000
+			run
+#endif
 			return True
 		, do
 			void $ redirect $ "file://" ++ path
