@@ -22,14 +22,14 @@ def = [notDirect $ command "rekey"
 	(paramOptional $ paramRepeating $ paramPair paramPath paramKey)
 	seek SectionPlumbing "change keys used for files"]
 
-seek :: [CommandSeek]
-seek = [withPairs start]
+seek :: CommandSeek
+seek = withPairs start
 
 start :: (FilePath, String) -> CommandStart
 start (file, keyname) = ifAnnexed file go stop
   where
 	newkey = fromMaybe (error "bad key") $ file2key keyname
-	go (oldkey, _)
+	go oldkey
 		| oldkey == newkey = stop
 		| otherwise = do
 			showStart "rekey" file
